@@ -9,7 +9,7 @@ static int irowslen = -1;    // -1 is for irows = NULL
 static uint16_t *high=NULL, *low=NULL;  // the group of each x item; a.k.a. which-group-am-I
 static int *restrict grp;    // TODO: eventually this can be made local for gforce as won't be needed globally when all functions here use gather
 static size_t highSize;
-static int shift, mask;
+static int bitshift, mask;
 static char *gx=NULL;
 
 static size_t nBatch, batchSize, lastBatchSize;
@@ -70,10 +70,10 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
   }
 
   int nb = nbit(ngrp-1);
-  shift = nb/2;    // /2 so that high and low can be uint16_t, and no limit (even for nb=4) to stress-test.
-  // shift=MAX(nb-8,0); if (shift>16) shift=nb/2;     // TODO: when we have stress-test off mode, do this
-  mask = (1<<shift)-1;
-  highSize = ((ngrp-1)>>shift) + 1;
+  bitshift = nb/2;    // /2 so that high and low can be uint16_t, and no limit (even for nb=4) to stress-test.
+  // bitshift=MAX(nb-8,0); if (bitshift>16) bitshift=nb/2;     // TODO: when we have stress-test off mode, do this
+  mask = (1<<bitshift)-1;
+  highSize = ((ngrp-1)>>bitshift) + 1;
 
   grp = (int *)R_alloc(nrow, sizeof(int));   // TODO: use malloc and made this local as not needed globally when all functions here use gather
                                              // maybe better to malloc to avoid R's heap. This grp isn't global, so it doesn't need to be R_alloc
@@ -86,8 +86,8 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
   // TODO: enable stress-test mode in tests only (#3205) which can be turned off by default in release to decrease overhead on small data
   //       if that is established to be biting (it may be fine).
   if (nBatch<1 || batchSize<1 || lastBatchSize<1) {
-    error(_("Internal error: nrow=%d  ngrp=%d  nbit=%d  shift=%d  highSize=%d  nBatch=%d  batchSize=%d  lastBatchSize=%d\n"),  // # nocov
-           nrow, ngrp, nb, shift, highSize, nBatch, batchSize, lastBatchSize);                                              // # nocov
+    error(_("Internal error: nrow=%d  ngrp=%d  nbit=%d  bitshift=%d  highSize=%d  nBatch=%d  batchSize=%d  lastBatchSize=%d\n"),  // # nocov
+           nrow, ngrp, nb, bitshift, highSize, nBatch, batchSize, lastBatchSize);                                              // # nocov
   }
   // initial population of g:
   #pragma omp parallel for num_threads(getDTthreads(ngrp, false))
@@ -108,9 +108,9 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
 
     const int *restrict op = INTEGER(o);  // o is a permutation of 1:nrow
     int nb = nbit(nrow-1);
-    int shift = MAX(nb-8, 0);  // TODO: experiment nb/2.  Here it doesn't have to be /2 currently.
-    int highSize = ((nrow-1)>>shift) + 1;
-    //Rprintf(_("When assigning grp[o] = g, highSize=%d  nb=%d  shift=%d  nBatch=%d\n"), highSize, nb, shift, nBatch);
+    int bitshift = MAX(nb-8, 0);  // TODO: experiment nb/2.  Here it doesn't have to be /2 currently.
+    int highSize = ((nrow-1)>>bitshift) + 1;
+    //Rprintf(_("When assigning grp[o] = g, highSize=%d  nb=%d  bitshift=%d  nBatch=%d\n"), highSize, nb, bitshift, nBatch);
     int *counts = calloc(nBatch*highSize, sizeof(int));  // TODO: cache-line align and make highSize a multiple of 64
     int *TMP   = malloc(nrow*2l*sizeof(int)); // must multiple the long int otherwise overflow may happen, #4295
     if (!counts || !TMP ) error(_("Internal error: Failed to allocate counts or TMP when assigning g in gforce"));
@@ -120,7 +120,7 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
       const int *my_o = op + b*batchSize;
       int *restrict my_counts = counts + b*highSize;
       for (int i=0; i<howMany; i++) {
-        const int w = (my_o[i]-1) >> shift;
+        const int w = (my_o[i]-1) >> bitshift;
         my_counts[w]++;
       }
       for (int i=0, cum=0; i<highSize; i++) {
@@ -131,7 +131,7 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
       const int *restrict my_g = grp + b*batchSize;
       int *restrict my_tmp = TMP + b*2*batchSize;
       for (int i=0; i<howMany; i++) {
-        const int w = (my_o[i]-1) >> shift;   // could use my_high but may as well use my_pg since we need my_pg anyway for the lower bits next too
+        const int w = (my_o[i]-1) >> bitshift;   // could use my_high but may as well use my_pg since we need my_pg anyway for the lower bits next too
         int *p = my_tmp + 2*my_counts[w]++;
         *p++ = my_o[i]-1;
         *p   = my_g[i];
@@ -172,7 +172,7 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
     const int *my_pg = gp + b*batchSize;
     const int howMany = b==nBatch-1 ? lastBatchSize : batchSize;
     for (int i=0; i<howMany; i++) {
-      const int w = my_pg[i] >> shift;
+      const int w = my_pg[i] >> bitshift;
       my_counts[w]++;
       my_high[i] = (uint16_t)w;  // reduce 4 bytes to 2
     }
@@ -185,7 +185,7 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
     int *restrict my_tmpcounts = tmpcounts + omp_get_thread_num()*highSize;
     memcpy(my_tmpcounts, my_counts, highSize*sizeof(int));
     for (int i=0; i<howMany; i++) {
-      const int w = my_pg[i] >> shift;   // could use my_high but may as well use my_pg since we need my_pg anyway for the lower bits next too
+      const int w = my_pg[i] >> bitshift;   // could use my_high but may as well use my_pg since we need my_pg anyway for the lower bits next too
       my_low[my_tmpcounts[w]++] = (uint16_t)(my_pg[i] & mask);
     }
     // counts is now cumulated within batch (with ending values) and we leave it that way
@@ -361,7 +361,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
     if (!anyNA) {
       #pragma omp parallel for num_threads(getDTthreads(highSize, false)) //schedule(dynamic,1)
       for (int h=0; h<highSize; h++) {   // very important that high is first loop here
-        int *restrict _ans = ansp + (h<<shift);
+        int *restrict _ans = ansp + (h<<bitshift);
         for (int b=0; b<nBatch; b++) {
           const int pos = counts[ b*highSize + h ];
           const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -378,7 +378,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
     } else {
       #pragma omp parallel for num_threads(getDTthreads(highSize, false))
       for (int h=0; h<highSize; h++) {
-        int *restrict _ans = ansp + (h<<shift);
+        int *restrict _ans = ansp + (h<<bitshift);
         for (int b=0; b<nBatch; b++) {
           const int pos = counts[ b*highSize + h ];
           const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -407,7 +407,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
       memset(ansp, 0, ngrp*sizeof(double));
       #pragma omp parallel for num_threads(getDTthreads(highSize, false))
       for (int h=0; h<highSize; h++) {
-        double *restrict _ans = ansp + (h<<shift);
+        double *restrict _ans = ansp + (h<<bitshift);
         for (int b=0; b<nBatch; b++) {
           const int pos = counts[ b*highSize + h ];
           const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -435,7 +435,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
       if (!narm || !anyNA) {
         #pragma omp parallel for num_threads(getDTthreads(highSize, false))
         for (int h=0; h<highSize; h++) {
-          double *restrict _ans = ansp + (h<<shift);
+          double *restrict _ans = ansp + (h<<bitshift);
           for (int b=0; b<nBatch; b++) {
             const int pos = counts[ b*highSize + h ];
             const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -450,7 +450,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
         // narm==true and anyNA==true
         #pragma omp parallel for num_threads(getDTthreads(highSize, false))
         for (int h=0; h<highSize; h++) {
-          double *restrict _ans = ansp + (h<<shift);
+          double *restrict _ans = ansp + (h<<bitshift);
           for (int b=0; b<nBatch; b++) {
             const int pos = counts[ b*highSize + h ];
             const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -471,7 +471,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
       if (!anyNA) {
         #pragma omp parallel for num_threads(getDTthreads(highSize, false))
         for (int h=0; h<highSize; h++) {
-          int64_t *restrict _ans = ansp + (h<<shift);
+          int64_t *restrict _ans = ansp + (h<<bitshift);
           for (int b=0; b<nBatch; b++) {
             const int pos = counts[ b*highSize + h ];
             const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -486,7 +486,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
         if (!narm) {
           #pragma omp parallel for num_threads(getDTthreads(highSize, false))
           for (int h=0; h<highSize; h++) {
-            int64_t *restrict _ans = ansp + (h<<shift);
+            int64_t *restrict _ans = ansp + (h<<bitshift);
             for (int b=0; b<nBatch; b++) {
               const int pos = counts[ b*highSize + h ];
               const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -506,7 +506,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
         } else {
           #pragma omp parallel for num_threads(getDTthreads(highSize, false))
           for (int h=0; h<highSize; h++) {
-            int64_t *restrict _ans = ansp + (h<<shift);
+            int64_t *restrict _ans = ansp + (h<<bitshift);
             for (int b=0; b<nBatch; b++) {
               const int pos = counts[ b*highSize + h ];
               const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -530,7 +530,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
     if (!narm || !anyNA) {
       #pragma omp parallel for num_threads(getDTthreads(highSize, false))
       for (int h=0; h<highSize; h++) {
-        Rcomplex *restrict _ans = ansp + (h<<shift);
+        Rcomplex *restrict _ans = ansp + (h<<bitshift);
         for (int b=0; b<nBatch; b++) {
           const int pos = counts[ b*highSize + h ];
           const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
@@ -546,7 +546,7 @@ SEXP gsum(SEXP x, SEXP narmArg, SEXP warnOverflowArg)
       // narm==true and anyNA==true
       #pragma omp parallel for num_threads(getDTthreads(highSize, false))
       for (int h=0; h<highSize; h++) {
-        Rcomplex *restrict _ans = ansp + (h<<shift);
+        Rcomplex *restrict _ans = ansp + (h<<bitshift);
         for (int b=0; b<nBatch; b++) {
           const int pos = counts[ b*highSize + h ];
           const int howMany = ((h==highSize-1) ? (b==nBatch-1?lastBatchSize:batchSize) : counts[ b*highSize + h + 1 ]) - pos;
